@@ -330,7 +330,18 @@ static int standby_resume_standby(struct node *node, unsigned me, unsigned la, b
 
 	fail_on_test(!util_interactive_ensure_power_state(node, me, la, interactive, CEC_OP_POWER_STATUS_ON));
 
+	/*
+	 * Some displays only accept Standby from the Active Source.
+	 * So make us the Active Source before sending Standby.
+	 */
+	if (is_tv(la, node->remote[la].prim_type)) {
+		announce("Sending Active Source message.");
+		cec_msg_init(&msg, me, la);
+		cec_msg_active_source(&msg, node->phys_addr);
+		fail_on_test(doioctl(node, CEC_TRANSMIT, &msg));
+	}
 	announce("Sending Standby message.");
+
 	cec_msg_init(&msg, me, la);
 	cec_msg_standby(&msg);
 	fail_on_test(!transmit_timeout(node, &msg));
@@ -396,13 +407,17 @@ static int standby_resume_active_source_nowake(struct node *node, unsigned me, u
 	cec_msg_active_source(&msg, node->phys_addr);
 	int res = doioctl(node, CEC_TRANSMIT, &msg);
 	fail_on_test(res && res != ENONET);
-	fail_on_test(wait_changing_power_status(node, me, la, new_status, unresponsive_time));
-	fail_on_test(new_status != CEC_OP_POWER_STATUS_STANDBY);
+	fail_on_test_v2_warn(node->remote[la].cec_version,
+			     wait_changing_power_status(node, me, la, new_status, unresponsive_time));
+	fail_on_test_v2_warn(node->remote[la].cec_version,
+			     new_status != CEC_OP_POWER_STATUS_STANDBY);
+	if (new_status != CEC_OP_POWER_STATUS_STANDBY)
+		return standby_resume_standby(node, me, la, interactive);
+
 	node->remote[la].in_standby = true;
 	if (unresponsive_time > 0)
 		warn("The device stayed correctly in standby, but became unresponsive for %d s.\n",
 		     unresponsive_time);
-
 	return 0;
 }
 
@@ -544,6 +559,16 @@ static int standby_resume_wakeup_text_view_on(struct node *node, unsigned me, un
 struct remote_subtest standby_resume_subtests[] = {
 	{ "Standby", CEC_LOG_ADDR_MASK_ALL, standby_resume_standby },
 	{ "Repeated Standby message does not wake up", CEC_LOG_ADDR_MASK_ALL, standby_resume_standby_toggle },
+	{ "Standby: Feature aborts unknown messages", CEC_LOG_ADDR_MASK_ALL, core_unknown, true },
+	{ "Standby: Feature aborts Abort message", CEC_LOG_ADDR_MASK_ALL, core_abort, true },
+	{ "Standby: Polling Message", CEC_LOG_ADDR_MASK_ALL, system_info_polling, true },
+	{ "Standby: Give Device Power Status", CEC_LOG_ADDR_MASK_ALL, power_status_give, true },
+	{ "Standby: Give Physical Address", CEC_LOG_ADDR_MASK_ALL, system_info_phys_addr, true },
+	{ "Standby: Give CEC Version", CEC_LOG_ADDR_MASK_ALL, system_info_version, true },
+	{ "Standby: Give Device Vendor ID", CEC_LOG_ADDR_MASK_ALL, vendor_specific_commands_id, true },
+	{ "Standby: Give OSD Name", CEC_LOG_ADDR_MASK_ALL, device_osd_transfer_give, true },
+	{ "Standby: Get Menu Language", CEC_LOG_ADDR_MASK_ALL, system_info_get_menu_lang, true },
+	{ "Standby: Give Device Features", CEC_LOG_ADDR_MASK_ALL, system_info_give_features, true },
 	{ "No wakeup on Active Source", CEC_LOG_ADDR_MASK_ALL, standby_resume_active_source_nowake },
 	{ "Wake up", CEC_LOG_ADDR_MASK_ALL, standby_resume_wakeup},
 	{ "Wake up TV on Image View On", CEC_LOG_ADDR_MASK_TV, standby_resume_wakeup_image_view_on },
