@@ -77,7 +77,7 @@
 #define PROGRAM_NAME	"dvbv5-daemon"
 
 const char *argp_program_version = PROGRAM_NAME " version " V4L_UTILS_VERSION;
-const char *argp_program_bug_address = "Mauro Carvalho Chehab <m.chehab@samsung.com>";
+const char *argp_program_bug_address = "Mauro Carvalho Chehab <mchehab@kernel.org>";
 
 static const char doc[] = N_(
 	"\nA DVB remote daemon using API version 5\n");
