@@ -495,8 +495,9 @@ void ApplicationWindow::setAudioBufferSize()
 void ApplicationWindow::ctrlEvent()
 {
 	v4l2_event ev;
+	int event_ret = 0;
 
-	while (dqevent(ev) == 0) {
+	while ((event_ret = dqevent(ev)) == 0) {
 		if (ev.type == V4L2_EVENT_SOURCE_CHANGE) {
 			m_genTab->sourceChange(ev);
 			continue;
@@ -551,6 +552,15 @@ void ApplicationWindow::ctrlEvent()
 			setString(ev.id, c.string);
 		free(c.string);
 	}
+
+	if (event_ret && errno == ENODEV) {
+		closeDevice();
+		if (m_capture) {
+			m_capture->stop();
+			delete m_capture;
+			m_capture = NULL;
+		}
+	}
 }
 
 void ApplicationWindow::newCaptureWin()
@@ -558,6 +568,7 @@ void ApplicationWindow::newCaptureWin()
 	if (m_capture != NULL) {
 		m_capture->stop();
 		delete m_capture;
+		m_capture = NULL;
 	}
 
 	switch (m_renderMethod) {
@@ -1135,7 +1146,7 @@ void ApplicationWindow::stopStreaming()
 	if (!m_genTab->isSDR() && m_genTab->isRadio())
 		return;
 
-	if (v4l_type_is_capture(g_type()))
+	if (v4l_type_is_capture(g_type()) && m_capture)
 		m_capture->stop();
 
 	m_snapshotAct->setDisabled(true);
@@ -1516,11 +1527,15 @@ void ApplicationWindow::capStart(bool start)
 	case V4L2_PIX_FMT_BGR32:
 	case V4L2_PIX_FMT_XRGB32:
 	case V4L2_PIX_FMT_XBGR32:
+	case V4L2_PIX_FMT_YUV32:
+	case V4L2_PIX_FMT_XYUV32:
+	case V4L2_PIX_FMT_VUYX32:
 		dstFmt = QImage::Format_RGB32;
 		break;
 	case V4L2_PIX_FMT_ARGB32:
 	case V4L2_PIX_FMT_ABGR32:
-	case V4L2_PIX_FMT_YUV32:
+	case V4L2_PIX_FMT_AYUV32:
+	case V4L2_PIX_FMT_VUYA32:
 		dstFmt = QImage::Format_ARGB32;
 		break;
 	case V4L2_PIX_FMT_INZI:
@@ -1557,8 +1572,10 @@ void ApplicationWindow::makeFullScreen(bool checked)
 void ApplicationWindow::closeDevice()
 {
 	stopAudio();
-	delete m_sigMapper;
-	m_sigMapper = NULL;
+	if (m_sigMapper) {
+		m_sigMapper->deleteLater();
+		m_sigMapper = NULL;
+	}
 	m_capStartAct->setEnabled(false);
 	m_capStartAct->setChecked(false);
 	m_capStepAct->setEnabled(false);
@@ -1575,7 +1592,7 @@ void ApplicationWindow::closeDevice()
 			m_outNotifier = NULL;
 		}
 		if (m_ctrlNotifier) {
-			delete m_ctrlNotifier;
+			m_ctrlNotifier->deleteLater();
 			m_ctrlNotifier = NULL;
 		}
 		delete [] m_frameData;
@@ -1701,7 +1718,7 @@ void ApplicationWindow::error(const QString &error)
 {
 	statusBar()->showMessage(error, 20000);
 	if (!error.isEmpty())
-		fprintf(stderr, "%s\n", error.toLatin1().data());
+		fprintf(stderr, "%s\n", error.toUtf8().data());
 }
 
 void ApplicationWindow::error(int err)
@@ -1736,6 +1753,7 @@ void ApplicationWindow::closeEvent(QCloseEvent *event)
 {
 	closeDevice();
 	delete m_capture;
+	m_capture = NULL;
 	event->accept();
 }
 
@@ -1776,7 +1794,7 @@ static bool processShortOption(const QStringList &args, int &i, QString &dev)
 		return false;
 	if (args[i].length() == 2) {
 		if (i + 1 >= args.size()) {
-			usageError(args[i].toLatin1());
+			usageError(args[i].toUtf8());
 			return false;
 		}
 		dev = args[++i];
@@ -1799,7 +1817,7 @@ static bool processLongOption(const QStringList &args, int &i, QString &dev)
 		return true;
 	}
 	if (i + 1 >= args.size()) {
-		usageError(args[i].toLatin1());
+		usageError(args[i].toUtf8());
 		return false;
 	}
 	dev = args[++i];
@@ -1853,7 +1871,7 @@ int main(int argc, char **argv)
 		} else if (args[i] == "-R" || args[i] == "--raw") {
 			raw = true;
 		} else {
-			printf("Invalid argument %s\n", args[i].toLatin1().data());
+			printf("Invalid argument %s\n", args[i].toUtf8().data());
 			return 0;
 		}
 	}
