@@ -50,13 +50,29 @@
 #endif
 
 extern bool show_info;
+extern bool show_colors;
 extern bool show_warnings;
+extern bool no_progress;
 extern bool exit_on_fail;
 extern bool exit_on_warn;
+extern bool is_vivid; // We're testing the vivid driver
 extern int kernel_version;
 extern int media_fd;
 extern unsigned warnings;
 
+enum poll_mode {
+	POLL_MODE_NONE,
+	POLL_MODE_SELECT,
+	POLL_MODE_EPOLL,
+};
+
+#define JPEG_ENCODER (1 << 0)
+#define JPEG_DECODER (1 << 1)
+#define STATEFUL_ENCODER (1 << 2)
+#define STATEFUL_DECODER (1 << 3)
+#define STATELESS_ENCODER (1 << 4)
+#define STATELESS_DECODER (1 << 5)
+
 struct test_query_ext_ctrl: v4l2_query_ext_ctrl {
 	__u64 menu_mask;
 };
@@ -68,7 +84,7 @@ typedef std::map<__u32, unsigned> frmsizes_count_map;
 
 struct base_node;
 
-#define V4L2_BUF_TYPE_LAST V4L2_BUF_TYPE_META_CAPTURE
+#define V4L2_BUF_TYPE_LAST V4L2_BUF_TYPE_META_OUTPUT
 
 struct base_node {
 	bool is_video;
@@ -97,7 +113,9 @@ struct base_node {
 	unsigned std_compound_controls;
 	unsigned priv_controls;
 	unsigned priv_compound_controls;
+	unsigned codec_mask;
 	__u32 media_version;
+	std::string media_bus_info;
 	media_entity_desc entity;
 	media_pad_desc *pads;
 	media_link_desc *links;
@@ -112,6 +130,8 @@ struct base_node {
 	int frame_interval_pad;
 	int enum_frame_interval_pad;
 	__u32 fbuf_caps;
+	__u32 buf_caps;
+	const char *bus_info;
 	pixfmt_map buftype_pixfmts[V4L2_BUF_TYPE_LAST + 1];
 	frmsizes_set frmsizes;
 	frmsizes_count_map frmsizes_count;
@@ -119,6 +139,7 @@ struct base_node {
 	__u32 valid_buftypes;
 	__u32 valid_buftype;
 	__u32 valid_memorytype;
+	bool supports_orphaned_bufs;
 };
 
 struct node : public base_node, public cv4l_fd {
@@ -126,19 +147,63 @@ struct node : public base_node, public cv4l_fd {
 
 	qctrl_map controls;
 	pixfmt_map buftype_pixfmts[V4L2_BUF_TYPE_LAST + 1];
+
+	bool inject_error(__u32 id)
+	{
+		v4l2_control ctrl = {
+			.id = id,
+		};
+
+		return is_vivid && !s_ctrl(ctrl);
+	}
 };
 
+class filehandles {
+public:
+	filehandles() {}
+	~filehandles()
+	{
+		for (std::set<int>::const_iterator iter = fhs.begin(); iter != fhs.end(); ++iter)
+			close(*iter);
+	}
+
+	int add(int fd)
+	{
+		if (fd >= 0)
+			fhs.insert(fd);
+		return fd;
+	}
+
+	void del(int fd)
+	{
+		if (fd >= 0) {
+			fhs.erase(fd);
+			close(fd);
+		}
+	}
+
+private:
+	std::set<int> fhs;
+};
+
+#define COLOR_GREEN(s) "\033[32m" s "\033[0m"
+#define COLOR_RED(s) "\033[1;31m" s "\033[0m"
+#define COLOR_BOLD(s) "\033[1m" s "\033[0m"
+
 #define info(fmt, args...) 					\
 	do {							\
 		if (show_info)					\
- 			printf("\t\tinfo: " fmt, ##args);	\
+			printf("\t\tinfo: " fmt, ##args);	\
 	} while (0)
 
 #define warn(fmt, args...) 					\
 	do {							\
 		warnings++;					\
 		if (show_warnings)				\
- 			printf("\t\twarn: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
+			printf("\t\t%s: %s(%d): " fmt,		\
+			       show_colors ?			\
+			       COLOR_BOLD("warn") : "warn",	\
+			       __FILE__, __LINE__, ##args);	\
 		if (exit_on_warn)				\
 			exit(1);				\
 	} while (0)
@@ -149,18 +214,20 @@ struct node : public base_node, public cv4l_fd {
 									\
 		if (!show) {						\
 			show = true;					\
-			warnings++;					\
-			if (show_warnings)				\
-				printf("\t\twarn: %s(%d): " fmt,	\
-					__FILE__, __LINE__, ##args); 	\
-			if (exit_on_warn)				\
-				exit(1);				\
+			warn(fmt, ##args);				\
 		}							\
 	} while (0)
 
+#define warn_on_test(test) 				\
+	do {						\
+		if (test)				\
+			warn("%s\n", #test);		\
+	} while (0)
+
 #define fail(fmt, args...) 						\
 ({ 									\
- 	printf("\t\tfail: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
+	printf("\t\t%s: %s(%d): " fmt, show_colors ?			\
+	       COLOR_RED("fail") : "fail", __FILE__, __LINE__, ##args);	\
 	if (exit_on_fail)						\
 		exit(1);						\
 	1;								\
@@ -191,15 +258,17 @@ int check_string(const char *s, size_t len);
 int check_ustring(const __u8 *s, int len);
 int check_0(const void *p, int len);
 int restoreFormat(struct node *node);
-void testNode(struct node &node, struct node &expbuf_node, media_type type,
-	      unsigned frame_count);
+void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_node, media_type type,
+	      unsigned frame_count, unsigned all_fmt_frame_count);
+std::string stream_from(std::string pixelformat, bool &use_hdr);
 
 // Media Controller ioctl tests
 int testMediaDeviceInfo(struct node *node);
 int testMediaTopology(struct node *node);
 int testMediaEnum(struct node *node);
 int testMediaSetupLink(struct node *node);
-void walkTopology(struct node &node, struct node &expbuf_node, unsigned frame_count);
+void walkTopology(struct node &node, struct node &expbuf_node,
+		  unsigned frame_count, unsigned all_fmt_frame_count);
 
 // Debug ioctl tests
 int testRegister(struct node *node);
@@ -262,10 +331,27 @@ int testReqBufs(struct node *node);
 int testReadWrite(struct node *node);
 int testExpBuf(struct node *node);
 int testBlockingWait(struct node *node);
-int testMmap(struct node *node, unsigned frame_count);
-int testUserPtr(struct node *node, unsigned frame_count);
-int testDmaBuf(struct node *expbuf_node, struct node *node, unsigned frame_count);
-void streamAllFormats(struct node *node);
+/*
+ * struct node node:
+ * 	the current media node being tested
+ *
+ * struct node node_m2m_cap:
+ * 	the capture device to be used when testing loopback or m2m, for
+ * 	instance, if the node we are testing is m2m, then node == node_m2m_cap,
+ * 	but if we have a topology like output->capture, node will be the output
+ * 	and node_m2m_cap will be the capture device that v4l2-compliance will
+ * 	use to test the loop
+ */
+int testMmap(struct node *node, struct node *node_m2m_cap, unsigned frame_count,
+	     enum poll_mode pollmode);
+int testUserPtr(struct node *node, struct node *node_m2m_cap,
+		unsigned frame_count, enum poll_mode pollmode);
+int testDmaBuf(struct node *expbuf_node, struct node *node,
+	       struct node *node_m2m_cap, unsigned frame_count,
+	       enum poll_mode pollmode);
+int testRequests(struct node *node, bool test_streaming);
+void streamAllFormats(struct node *node, unsigned frame_count);
+void streamM2MAllFormats(struct node *node, unsigned frame_count);
 
 // Color tests
 int testColorsAllFormats(struct node *node, unsigned component,
