@@ -199,6 +199,12 @@ static int checkTimings(struct node *node, bool has_timings, bool is_input)
 		fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &enumtimings.timings));
 		fail_on_test(check_0(enumtimings.timings.bt.reserved,
 				     sizeof(enumtimings.timings.bt.reserved)));
+		
+		struct v4l2_dv_timings g_timings;
+		fail_on_test(doioctl(node, VIDIOC_G_DV_TIMINGS, &g_timings));
+		fail_on_test(g_timings.bt.width != enumtimings.timings.bt.width);
+		fail_on_test(g_timings.bt.height != enumtimings.timings.bt.height);
+
 		if (node->is_vbi)
 			continue;
 		fmt.type = type;
@@ -210,6 +216,24 @@ static int checkTimings(struct node *node, bool has_timings, bool is_input)
 			fail_on_test(field == V4L2_FIELD_NONE);
 		else
 			fail_on_test(field != V4L2_FIELD_NONE);
+
+		unsigned factor = V4L2_FIELD_HAS_T_OR_B(field) ? 2 : 1;
+
+		// Test if the new format after setting new timings is either too small
+		// or more than 1.5 times the width/height. This allows for some
+		// adjustments to be made to the format, but still detect if the
+		// format isn't updated by the driver.
+		if (is_mplane) {
+			fail_on_test(fmt.fmt.pix_mp.width < enumtimings.timings.bt.width);
+			fail_on_test(fmt.fmt.pix_mp.width >= enumtimings.timings.bt.width * 1.5);
+			fail_on_test(fmt.fmt.pix_mp.height * factor < enumtimings.timings.bt.height);
+			fail_on_test(fmt.fmt.pix_mp.height * factor >= enumtimings.timings.bt.height * 1.5);
+		} else {
+			fail_on_test(fmt.fmt.pix.width < enumtimings.timings.bt.width);
+			fail_on_test(fmt.fmt.pix.width >= enumtimings.timings.bt.width * 1.5);
+			fail_on_test(fmt.fmt.pix.height * factor < enumtimings.timings.bt.height);
+			fail_on_test(fmt.fmt.pix.height * factor >= enumtimings.timings.bt.height * 1.5);
+		}
 	}
 	if (i == 0 && has_timings)
 		return fail("TIMINGS cap set, but no timings can be enumerated\n");
@@ -231,12 +255,127 @@ static int checkTimings(struct node *node, bool has_timings, bool is_input)
 	return 0;
 }
 
+static int checkSubDevEnumTimings(struct node *node, __u32 pad)
+{
+	struct v4l2_enum_dv_timings enumtimings;
+	struct v4l2_dv_timings timings;
+	bool found_fmt_mismatch = false;
+	bool has_timings;
+	int ret;
+
+	memset(&timings, 0xff, sizeof(timings));
+	ret = doioctl(node, VIDIOC_G_DV_TIMINGS, &timings);
+	has_timings = ret != ENOTTY;
+	fail_on_test(has_timings && ret);
+	if (has_timings) {
+		fail_on_test(check_0(timings.bt.reserved, sizeof(timings.bt.reserved)));
+		memset(timings.bt.reserved, 0xff, sizeof(timings.bt.reserved));
+		fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &timings));
+		fail_on_test(check_0(timings.bt.reserved, sizeof(timings.bt.reserved)));
+	} else {
+		fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &timings) != ENOTTY);
+		fail_on_test(doioctl(node, VIDIOC_QUERY_DV_TIMINGS, &timings) != ENOTTY);
+	}
+
+	for (unsigned i = 0; ; i++) {
+		memset(&enumtimings, 0xff, sizeof(enumtimings));
+
+		enumtimings.index = i;
+		enumtimings.pad = pad;
+		ret = doioctl(node, VIDIOC_ENUM_DV_TIMINGS, &enumtimings);
+		fail_on_test(ret != ENOTTY && !has_timings);
+		fail_on_test(ret == ENOTTY && has_timings);
+		if (ret == ENOTTY)
+			return ret;
+		fail_on_test(ret && ret != EINVAL);
+		fail_on_test(ret && !i);
+		if (ret)
+			break;
+		fail_on_test(check_0(enumtimings.reserved, sizeof(enumtimings.reserved)));
+		fail_on_test(check_0(enumtimings.timings.bt.reserved,
+				     sizeof(enumtimings.timings.bt.reserved)));
+		fail_on_test(enumtimings.index != i);
+		fail_on_test(enumtimings.pad != pad);
+		memset(enumtimings.timings.bt.reserved, 0xff,
+		       sizeof(enumtimings.timings.bt.reserved));
+		fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &enumtimings.timings));
+		fail_on_test(check_0(enumtimings.timings.bt.reserved,
+				     sizeof(enumtimings.timings.bt.reserved)));
+
+		struct v4l2_dv_timings g_timings;
+
+		fail_on_test(doioctl(node, VIDIOC_G_DV_TIMINGS, &g_timings));
+		fail_on_test(g_timings.bt.width != enumtimings.timings.bt.width);
+		fail_on_test(g_timings.bt.height != enumtimings.timings.bt.height);
+
+		if (found_fmt_mismatch)
+			continue;
+
+		struct v4l2_subdev_format fmt;
+
+		memset(&fmt, 0, sizeof(fmt));
+		fmt.which = V4L2_SUBDEV_FORMAT_ACTIVE;
+
+		// Test if the new format after setting new timings is either too small
+		// or more than 1.5 times the width/height. This allows for some
+		// adjustments to be made to the format, but still detect if the
+		// format isn't updated by the driver.
+		if (!doioctl(node, VIDIOC_SUBDEV_G_FMT, &fmt)) {
+			if (V4L2_FIELD_HAS_T_OR_B(fmt.format.field))
+				fmt.format.height *= 2;
+
+			if (fmt.format.width < enumtimings.timings.bt.width) {
+				warn("Active width for pad 0 is %u, which is less than the timings width of %u\n",
+				     fmt.format.width, enumtimings.timings.bt.width);
+				found_fmt_mismatch = true;
+			}
+			if (fmt.format.width >= enumtimings.timings.bt.width * 1.5) {
+				warn("Active width for pad 0 is %u, which is more than 1.5 * the timings width of %u\n",
+				     fmt.format.width, enumtimings.timings.bt.width);
+				found_fmt_mismatch = true;
+			}
+			if (fmt.format.height < enumtimings.timings.bt.height) {
+				warn("Active height for pad 0 is %u, which is less than the timings height of %u\n",
+				     fmt.format.height, enumtimings.timings.bt.height);
+				found_fmt_mismatch = true;
+			}
+			if (fmt.format.height >= enumtimings.timings.bt.height * 1.5) {
+				warn("Active height for pad 0 is %u, which is more than 1.5 * the timings height of %u\n",
+				     fmt.format.height, enumtimings.timings.bt.height);
+				found_fmt_mismatch = true;
+			}
+		}
+	}
+	enumtimings.pad = node->entity.pads;
+	enumtimings.index = 0;
+	fail_on_test(doioctl(node, VIDIOC_ENUM_DV_TIMINGS, &enumtimings) != EINVAL);
+
+	fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &timings));
+	memset(&timings, 0xff, sizeof(timings));
+	ret = doioctl(node, VIDIOC_QUERY_DV_TIMINGS, &timings);
+	fail_on_test(ret == ENOTTY);
+	fail_on_test(ret == ENODATA);
+	if (!ret || ret == ERANGE)
+		fail_on_test(check_0(timings.bt.reserved, sizeof(timings.bt.reserved)));
+	return 0;
+}
+
 int testTimings(struct node *node)
 {
 	int ret;
 	unsigned i, o;
 	bool has_timings = false;
 
+	if (node->is_subdev()) {
+		for (unsigned pad = 0; pad < node->entity.pads; pad++) {
+			ret = checkSubDevEnumTimings(node, pad);
+			if (!ret)
+				has_timings = true;
+			fail_on_test(ret && ret != ENOTTY);
+		}
+		return has_timings ? 0 : ENOTTY;
+	}
+
 	for (i = 0; i < node->inputs; i++) {
 		struct v4l2_input input;
 
@@ -271,14 +410,16 @@ int testTimings(struct node *node)
 	return has_timings ? 0 : ENOTTY;
 }
 
-static int checkTimingsCap(struct node *node, bool has_timings)
+static int checkTimingsCap(struct node *node, unsigned pad, bool has_timings)
 {
 	struct v4l2_dv_timings_cap timingscap;
 	int ret;
 
 	memset(&timingscap, 0xff, sizeof(timingscap));
-	timingscap.pad = 0;
+	timingscap.pad = pad;
 	ret = doioctl(node, VIDIOC_DV_TIMINGS_CAP, &timingscap);
+	if (node->is_subdev() && ret == ENOTTY)
+		return ENOTTY;
 	if (ret && has_timings)
 		return fail("TIMINGS cap set, but could not get timings caps\n");
 	if (!ret && !has_timings)
@@ -287,12 +428,20 @@ static int checkTimingsCap(struct node *node, bool has_timings)
 		return 0;
 	if (check_0(timingscap.reserved, sizeof(timingscap.reserved)))
 		return fail("reserved not zeroed\n");
+	fail_on_test(timingscap.pad != pad);
 	fail_on_test(timingscap.type != V4L2_DV_BT_656_1120);
 	if (check_0(timingscap.bt.reserved, sizeof(timingscap.bt.reserved)))
 		return fail("reserved not zeroed\n");
 	fail_on_test(timingscap.bt.min_width > timingscap.bt.max_width);
 	fail_on_test(timingscap.bt.min_height > timingscap.bt.max_height);
 	fail_on_test(timingscap.bt.min_pixelclock > timingscap.bt.max_pixelclock);
+
+	if (!node->is_subdev())
+		return 0;
+
+	memset(&timingscap, 0, sizeof(timingscap));
+	timingscap.pad = node->is_subdev() ? node->entity.pads : 1;
+	fail_on_test(doioctl(node, VIDIOC_DV_TIMINGS_CAP, &timingscap) != EINVAL);
 	return 0;
 }
 
@@ -302,6 +451,20 @@ int testTimingsCap(struct node *node)
 	unsigned i, o;
 	bool has_timings = false;
 
+	if (node->is_subdev()) {
+		for (unsigned pad = 0; pad < node->entity.pads; pad++) {
+			bool is_input = node->pads[pad].flags & MEDIA_PAD_FL_SINK;
+
+			ret = checkTimingsCap(node, pad, true);
+			if (ret && ret != ENOTTY)
+				return fail("EDID check failed for %s pad %u.\n",
+					    is_input ? "sink" : "source", pad);
+			if (!ret)
+				has_timings = true;
+		}
+		return has_timings ? 0 : ENOTTY;
+	}
+
 	for (i = 0; i < node->inputs; i++) {
 		struct v4l2_input input;
 
@@ -314,7 +477,7 @@ int testTimingsCap(struct node *node)
 			return fail("could not select input %d.\n", i);
 		if (input.capabilities & V4L2_IN_CAP_DV_TIMINGS)
 			has_timings = true;
-		if (checkTimingsCap(node, input.capabilities & V4L2_IN_CAP_DV_TIMINGS))
+		if (checkTimingsCap(node, 0, input.capabilities & V4L2_IN_CAP_DV_TIMINGS))
 			return fail("Timings cap failed for input %d.\n", i);
 	}
 
@@ -330,7 +493,7 @@ int testTimingsCap(struct node *node)
 			return fail("could not select output %d.\n", o);
 		if (output.capabilities & V4L2_OUT_CAP_DV_TIMINGS)
 			has_timings = true;
-		if (checkTimingsCap(node, output.capabilities & V4L2_OUT_CAP_DV_TIMINGS))
+		if (checkTimingsCap(node, 0, output.capabilities & V4L2_OUT_CAP_DV_TIMINGS))
 			return fail("Timings cap check failed for output %d.\n", o);
 	}
 	return has_timings ? 0 : ENOTTY;
@@ -354,7 +517,7 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 		memset(&edid, 0, sizeof(edid));
 		edid.pad = pad;
 		fail_on_test(doioctl(node, VIDIOC_S_EDID, &edid) != ENOTTY);
-		return 0;
+		return ENOTTY;
 	}
 	has_edid = ret == 0;
 	fail_on_test(ret && ret != EINVAL);
@@ -362,6 +525,7 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 	fail_on_test(edid.start_block);
 	fail_on_test(edid.blocks > 256);
 	fail_on_test(!has_edid && edid.blocks);
+	fail_on_test(edid.pad != pad);
 	blocks = edid.blocks;
 	edid.edid = data;
 	ret = doioctl(node, VIDIOC_G_EDID, &edid);
@@ -388,13 +552,18 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 		fail_on_test(ret && ret != ENODATA);
 		if (!ret)
 			fail_on_test(edid.blocks != blocks);
+		edid.pad = ~0;
+		fail_on_test(doioctl(node, VIDIOC_G_EDID, &edid) != EINVAL);
+		edid.pad = pad;
 	}
 
 	memset(edid.reserved, 0xff, sizeof(edid.reserved));
 	edid.blocks = blocks;
 	ret = doioctl(node, VIDIOC_S_EDID, &edid);
-	if (ret == ENOTTY)
+	if (ret == ENOTTY) {
+		fail_on_test(is_input);
 		return 0;
+	}
 	fail_on_test(!is_input);
 	if (!has_edid) {
 		fail_on_test(ret != EINVAL);
@@ -402,12 +571,26 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 	}
 	fail_on_test(ret);
 	fail_on_test(check_0(edid.reserved, sizeof(edid.reserved)));
-	if (blocks == 256)
-		return 0;
-	edid.blocks = 256;
+	edid.blocks = blocks;
+	edid.pad = ~0;
 	ret = doioctl(node, VIDIOC_S_EDID, &edid);
-	fail_on_test(ret != E2BIG);
-	fail_on_test(edid.blocks == 0 || edid.blocks >= 256);
+	fail_on_test(ret != EINVAL);
+	if (blocks < 256) {
+		edid.blocks = 256;
+		edid.pad = pad;
+		ret = doioctl(node, VIDIOC_S_EDID, &edid);
+		fail_on_test(ret != E2BIG);
+		fail_on_test(edid.blocks == 0 || edid.blocks >= 256);
+		fail_on_test(edid.pad != pad);
+	}
+	edid.blocks = blocks;
+	edid.pad = pad;
+	edid.edid = NULL;
+	ret = doioctl(node, VIDIOC_S_EDID, &edid);
+	fail_on_test(ret != EFAULT);
+	edid.edid = (__u8 *)0xdeadbeef;
+	ret = doioctl(node, VIDIOC_S_EDID, &edid);
+	fail_on_test(ret != EFAULT);
 	return 0;
 }
 
@@ -417,6 +600,20 @@ int testEdid(struct node *node)
 	unsigned i, o;
 	int ret;
 
+	if (node->is_subdev()) {
+		for (unsigned pad = 0; pad < node->entity.pads; pad++) {
+			bool is_input = node->pads[pad].flags & MEDIA_PAD_FL_SOURCE;
+
+			ret = checkEdid(node, pad, is_input);
+			if (ret && ret != ENOTTY)
+				return fail("EDID check failed for %s pad %u.\n",
+					    is_input ? "source" : "sink", pad);
+			if (!ret)
+				has_edid = true;
+		}
+		return has_edid ? 0 : ENOTTY;
+	}
+
 	for (i = 0; i < node->inputs; i++) {
 		struct v4l2_input input;
 
@@ -427,10 +624,11 @@ int testEdid(struct node *node)
 		ret = doioctl(node, VIDIOC_S_INPUT, &input.index);
 		if (ret)
 			return fail("could not select input %d.\n", i);
-		if (checkEdid(node, i, true))
+		ret = checkEdid(node, i, true);
+		if (ret && ret != ENOTTY)
 			return fail("EDID check failed for input %d.\n", i);
 		if (input.capabilities & V4L2_IN_CAP_DV_TIMINGS)
-			has_edid = true;
+			has_edid = !ret ? true : has_edid;
 	}
 
 	for (o = 0; o < node->outputs; o++) {
@@ -443,10 +641,11 @@ int testEdid(struct node *node)
 		ret = doioctl(node, VIDIOC_S_OUTPUT, &output.index);
 		if (ret)
 			return fail("could not select output %d.\n", o);
-		if (checkEdid(node, o, false))
+		ret = checkEdid(node, o, false);
+		if (ret && ret != ENOTTY)
 			return fail("EDID check failed for output %d.\n", o);
 		if (output.capabilities & V4L2_OUT_CAP_DV_TIMINGS)
-			has_edid = true;
+			has_edid = !ret ? true : has_edid;
 	}
 	return has_edid ? 0 : ENOTTY;
 }
